#!/usr/bin/env bash

# ********** Please don't edit this file!
# ********** It has been generated automatically by dae_devops version 0.5.1.
# ********** For repository_name dls-mainiac

me=${BASH_SOURCE}
echo "${me}: installing 0 dependencies for repository_name dls-mainiac"

function __install {
    echo ${me}: "$@"
    "$@"
}


# dae_devops_fingerprint 32c4a266f1c1cb7a1043d0bdf2548738